/*
 *  Copyright (C) 2015  Thadeu Lima de Souza Cascardo <cascardo@cascardo.eti.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "rendimento.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include "cmd.h"
#include "list.h"
#include "util.h"
#include "totais.h"

void rendimento_free(void *pointer)
{
	struct rendimento *rendimento = pointer;
	if (rendimento->cnpj)
		free(rendimento->cnpj);
	if (rendimento->nome)
		free(rendimento->nome);
	if (rendimento->saida)
		free(rendimento->saida);
	free(rendimento);
}

static int rendimento_cmp(void *p1, void *p2)
{
	struct rendimento *r1 = p1;
	struct rendimento *r2 = p2;
	/* O rendimento maior vem primeiro. */
	if (r1->rendimento > r2->rendimento)
		return -1;
	else if (r1->rendimento < r2->rendimento)
		return 1;
	return 0;
}

static struct rendimento * rendimento_new(char **args)
{
	struct rendimento *rendimento;
	int r = 0;
	rendimento = malloc(sizeof(*rendimento));
	rendimento->cnpj = strdup(args[1]);
	rendimento->nome = strdup(args[2]);
	rendimento->saida = strdup(args[8]);
	r += set_llong(args[3], &rendimento->rendimento);
	r += set_llong(args[4], &rendimento->previdencia);
	r += set_llong(args[5], &rendimento->decimoterceiro);
	r += set_llong(args[6], &rendimento->imposto);
	r += set_llong(args[7], &rendimento->imposto_13o);
	if (!rendimento->cnpj || !rendimento->nome || !rendimento->saida) {
		rendimento_free(rendimento);
		return NULL;
	}
	if (r < 0 || rendimento->rendimento < 0 || rendimento->previdencia < 0 ||
	    rendimento->decimoterceiro < 0 || rendimento->imposto < 0 ||
	    rendimento->imposto_13o < 0) {
		rendimento_free(rendimento);
		return NULL;
	}
	return rendimento;
}

static int run_rendimento(struct declaracao *dec, char **args, int argc)
{
	struct rendimento *rendimento;
	int r;
	if (argc != 9)
		return -EINVAL;
	rendimento = rendimento_new(args);
	if (!rendimento)
		return -ENOMEM;
	r = list_insert_ordered(&dec->rendimento, rendimento, rendimento_cmp);
	if (r < 0) {
		rendimento_free(rendimento);
		return r;
	}
	r = totais_add(dec, "RENDTRIB", rendimento->rendimento);
	r += totais_add(dec, "RENDPJ", rendimento->rendimento);
	r += totais_add(dec, "RENDPJTIT", rendimento->rendimento);
	r += totais_add(dec, "INSS", rendimento->previdencia);
	r += totais_add(dec, "DECIMOTERCEIRO", rendimento->decimoterceiro);
	r += totais_add(dec, "EXCLUSIVOS", rendimento->decimoterceiro);
	r += totais_add(dec, "EXCLUSIVOSTIT", rendimento->decimoterceiro);
	if (r) {
		rendimento_free(rendimento);
		return r;
	}
	return 0;
}

void rendimento_salva(struct declaracao *dec, FILE *f)
{
	int i;
	struct rendimento *j;
	for (i = 0; j = list_get(dec->rendimento, i); i++)
		fprintf(f, "rendimento \"%s\" \"%s\" %lld %lld %lld %lld %lld \"%s\"\n",
			j->cnpj, j->nome, j->rendimento, j->previdencia,
			j->decimoterceiro, j->imposto, j->imposto_13o, j->saida);
}

static struct cmd cmd_rendimento = {
	.name = "rendimento",
	.run = run_rendimento,
};

int rendimento_cmd_init(void)
{
	cmd_add(&cmd_rendimento);
	return 0;
}

char * rendimento_cnpj_ordenado(struct declaracao *dec, int i)
{
	struct rendimento *rendimento;
	rendimento = list_get(dec->rendimento, i);
	if (!rendimento)
		return "";
	return rendimento->cnpj;
}
